/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "operator/operator.pmh"

#define FE_SAC_DEBUG	FALSE

namespace fe
{

void SurfaceAttrCreateOp::initialize(void)
{
	catalog<String>("icon")="FE_alpha";

	catalog<String>("Name")="attribute";

	catalog<String>("Class")="Point";
	catalog<String>("Class","choice:0")="Point";
	catalog<String>("Class","choice:1")="Primitive";

	catalog<String>("Type")="Float";
	catalog<String>("Type","choice:0")="Float";
	catalog<String>("Type","choice:1")="Integer";
	catalog<String>("Type","choice:2")="Vector";
	catalog<String>("Type","choice:3")="String";

	catalog<String>("Default")="default";

	catalog<bool>("writeValues")=false;
	catalog<String>("writeValues","label")="Write Values";

	catalog<String>("Value");
	catalog<String>("Value","enabler")="writeValues";

	catalog< sp<Component> >("Input Surface");
}

void SurfaceAttrCreateOp::handle(Record& a_rSignal)
{
#if FE_SAC_DEBUG
	feLog("SurfaceAttrCreateOp::handle\n");
#endif

	const String attrName=catalog<String>("Name");
	if(attrName.empty())
	{
		feLog("SurfaceAttrCreateOp::handle no name\n");
		return;
	}
	const String attrClass=catalog<String>("Class");
	const String attrType=catalog<String>("Type");
	const String& attrDefault=catalog<String>("Default");

	const BWORD attrWriteValues=catalog<bool>("writeValues");
	const String& attrValue=catalog<String>("Value");

	sp<SurfaceAccessorI> spOutputElement;
	accessOutput(spOutputElement,a_rSignal,
			attrClass=="Point"? e_point: e_primitive,
			attrName,e_quiet,e_refuseMissing);
	if(spOutputElement.isValid())
	{
		if(!attrWriteValues)
		{
			//* don't clobber existing data
			return;
		}
	}
	else
	{
		//* create new attribute
		if(!accessOutput(spOutputElement,a_rSignal,
				attrClass=="Point"? e_point: e_primitive,attrName)) return;
	}

	const String& attrSet=(attrWriteValues)? attrValue: attrDefault;

	I32 elementCount=spOutputElement->count();

#if FE_SAC_DEBUG
	feLog("SurfaceAttrCreateOp::handle elementCount=%d\n",elementCount);
#endif

	if(attrType=="String")
	{
		for(I32 index=0;index<elementCount;index++)
		{
#if FE_SAC_DEBUG
			feLog("SurfaceAttrCreateOp::handle set %d \"%s\"\n",
					index,attrSet.c_str());
#endif
			spOutputElement->set(index,0,attrSet);
		}
	}
	else if(attrType=="I32")
	{
		const I32 value=atoi(attrSet.c_str());
		for(I32 index=0;index<elementCount;index++)
		{
#if FE_SAC_DEBUG
			feLog("SurfaceAttrCreateOp::handle set %d %d\n",
					index,value);
#endif
			spOutputElement->set(index,0,value);
		}
	}
	else if(attrType=="Real")
	{
		const Real value=atof(attrSet.c_str());
		for(I32 index=0;index<elementCount;index++)
		{
#if FE_SAC_DEBUG
			feLog("SurfaceAttrCreateOp::handle set %d %.6G\n",
					index,value);
#endif
			spOutputElement->set(index,0,value);
		}
	}
	else if(attrType=="SpatialVector")
	{
		Real value0;
		Real value1;
		Real value2;

		sscanf(attrSet.c_str(),"%.6G %.6G %.6G",
				&value0,&value1,&value2);

		const SpatialVector defaultVector(value0,value1,value2);

		for(I32 index=0;index<elementCount;index++)
		{
#if FE_SAC_DEBUG
			feLog("SurfaceAttrCreateOp::handle set %d %s\n",
					index,c_print(defaultVector));
#endif
			spOutputElement->set(index,0,defaultVector);
		}
	}

#if FE_SAC_DEBUG
	feLog("SurfaceAttrCreateOp::handle done\n");
#endif
}

} // namespace
